#include "defs.h"
#include <unistd.h>
#include <iostream>
#include "SDL.h"
#include "SDL_image.h"


SDL_Surface *surface;
SDL_Joystick *joystick;
SDL_Surface *imgBackground, *imgSelection;
Uint32 milliseconds;
int quit;

// Joystick states (1=pressed)
int up, down, left, right, upleft, upright, downleft, downright, start;

// Location of the squares that represent joystick positions
//   rX[Y]:
//   - X: 1=upleft, 2=up, 3=upright, 4=right, 6=downright, ...
//   - Y: 0=case1, 1=case2, 2=case3, ...
SDL_Rect r1[6], r2[6], r3[6], r4[6], r5[6], r6[6], r7[6], r8[6];

// Joystick states for each case
//   jX[Y]:
//   - X: 1=upleft, 2=up, 3=upright, 4=right, 6=downright, ...
//   - Y: 0=case1, 1=case2, 2=case3, ...
int j1[6], j2[6], j3[6], j4[6], j5[6], j6[6], j7[6], j8[6];

void InitSDL();
void CleanSDL();
void HandleInput();
void Update();
void Draw();
#ifdef TARGET_GP2X
void KeyDown(const Uint8& key);
void KeyUp(const Uint8& key);
#else
void KeyDown(const SDLKey& key);
void KeyUp(const SDLKey& key);
#endif



int main ( int argc, char** argv )
{
    int i;

    InitSDL();

    quit=0;
    up=down=left=right=upleft=upright=downleft=downright=start=0;

    milliseconds=SDL_GetTicks();

    for (i=0; i<8; i++) j1[i]=j2[i]=j3[i]=j4[i]=j5[i]=j6[i]=j7[i]=j8[i]=0;

    r1[0].x=22; r1[0].y=5; r2[0].x=41; r2[0].y=5; r3[0].x=60; r3[0].y=5; r4[0].x=60; r4[0].y=24; r5[0].x=60; r5[0].y=43; r6[0].x=41; r6[0].y=43; r7[0].x=22; r7[0].y=43; r8[0].x=22; r8[0].y=24;
    r1[1].x=133; r1[1].y=5; r2[1].x=152; r2[1].y=5; r3[1].x=171; r3[1].y=5; r4[1].x=171; r4[1].y=24; r5[1].x=171; r5[1].y=43; r6[1].x=152; r6[1].y=43; r7[1].x=133; r7[1].y=43; r8[1].x=133; r8[1].y=24;
    r1[2].x=243; r1[2].y=5; r2[2].x=262; r2[2].y=5; r3[2].x=281; r3[2].y=5; r4[2].x=281; r4[2].y=24; r5[2].x=281; r5[2].y=43; r6[2].x=262; r6[2].y=43; r7[2].x=243; r7[2].y=43; r8[2].x=243; r8[2].y=24;
    r1[3].x=22; r1[3].y=116; r2[3].x=41; r2[3].y=116; r3[3].x=60; r3[3].y=116; r4[3].x=60; r4[3].y=135; r5[3].x=60; r5[3].y=154; r6[3].x=41; r6[3].y=154; r7[3].x=22; r7[3].y=154; r8[3].x=22; r8[3].y=135;
    r1[4].x=133; r1[4].y=116; r2[4].x=152; r2[4].y=116; r3[4].x=171; r3[4].y=116; r4[4].x=171; r4[4].y=135; r5[4].x=171; r5[4].y=154; r6[4].x=152; r6[4].y=154; r7[4].x=133; r7[4].y=154; r8[4].x=133; r8[4].y=135;
    r1[5].x=243; r1[5].y=116; r2[5].x=262; r2[5].y=116; r3[5].x=281; r3[5].y=116; r4[5].x=281; r4[5].y=135; r5[5].x=281; r5[5].y=154; r6[5].x=262; r6[5].y=154; r7[5].x=243; r7[5].y=154; r8[5].x=243; r8[5].y=135;

    while (!quit)
    {
        if (SDL_GetTicks()-milliseconds>SPF)
        {
            HandleInput();
            Update();
            Draw();
            milliseconds=SDL_GetTicks();
        }
    }

    CleanSDL();

#ifdef TARGET_GP2X
    chdir("/usr/gp2x");
    execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif

    return 0;
}


void HandleInput()
{
    SDL_Event event;

    while(SDL_PollEvent(&event))
    {
        switch(event.type)
        {
#ifdef TARGET_GP2X
            case SDL_JOYBUTTONDOWN: KeyDown(event.jbutton.button); break;
            case SDL_JOYBUTTONUP: KeyUp(event.jbutton.button); break;
#else
            case SDL_KEYDOWN: KeyDown(event.key.keysym.sym); break;
            case SDL_KEYUP: KeyUp(event.key.keysym.sym); break;
            case SDL_QUIT: quit = true; break;
#endif
        }
    }
}


void Update()
{
    if (start) quit=true;

    // Case 1
    j1[0]=upleft;
    j2[0]=up;
    j3[0]=upright;
    j4[0]=right;
    j5[0]=downright;
    j6[0]=down;
    j7[0]=downleft;
    j8[0]=left;

    // Case 2
    j1[1]=upleft && !left && !up;
    j2[1]=up || (up&&upleft) || (up&&upright);
    j3[1]=upright && !right && !up;
    j4[1]=right || (right&&upright) || (right&&downright);
    j5[1]=downright && !right && !down;
    j6[1]=down || (down&&downleft) || (down&&downright);
    j7[1]=downleft && !left && !down;
    j8[1]=left || (left&&upleft) || (left&&downleft);

    // Case 3
    j1[2]=upleft || (upleft&&up) || (upleft&&left);
    j2[2]=up && !upleft && !upright;
    j3[2]=upright || (upright&&up) || (upright&&right);
    j4[2]=right && !upright && !downright;
    j5[2]=downright || (downright&&right) || (downright&&down);
    j6[2]=down && !downleft && !downright;
    j7[2]=downleft || (downleft&&down) || (downleft&&left);
    j8[2]=left && !downleft && !upleft;

    // Case 4
    j1[3]=upleft && !up && !left;
    j2[3]=up && !upleft && !upright;
    j3[3]=upright && !up && !right;
    j4[3]=right && !upright && !downright;
    j5[3]=downright && !down && !right;
    j6[3]=down && !downleft && !downright;
    j7[3]=downleft && !down && !left;
    j8[3]=left && !upleft && !downleft;

    // Case 5
    j2[4]=(upleft&&!left) || (upleft&&up) || up || (up&&upright);
    j4[4]=(upright&&!up) || (upright&&right) || right || (right&&downright);
    j6[4]=(downright&&!right) || (downright&&down) || down || (down&&downleft);
    j8[4]=(downleft&&!down) || (downleft&&left) || left || (left&&upleft);

    // Case 6
    j2[5]=(up&&!upleft) || (up&&upright) || upright || (upright&&right);
    j4[5]=(right&&!upright) || (right&&downright) || downright || (downright&&down);
    j6[5]=(down&&!downright) || (down&&downleft) || downleft || (downleft&&left);
    j8[5]=(left&&!downleft) || (left&&upleft) || upleft || (upleft&&up);
}


void Draw()
{
    int i;

    SDL_BlitSurface(imgBackground, NULL, surface, NULL);
    for (i=0; i<6; i++)
    {
        if (j1[i]) SDL_BlitSurface(imgSelection, NULL, surface, &(r1[i]));
        if (j2[i]) SDL_BlitSurface(imgSelection, NULL, surface, &(r2[i]));
        if (j3[i]) SDL_BlitSurface(imgSelection, NULL, surface, &(r3[i]));
        if (j4[i]) SDL_BlitSurface(imgSelection, NULL, surface, &(r4[i]));
        if (j5[i]) SDL_BlitSurface(imgSelection, NULL, surface, &(r5[i]));
        if (j6[i]) SDL_BlitSurface(imgSelection, NULL, surface, &(r6[i]));
        if (j7[i]) SDL_BlitSurface(imgSelection, NULL, surface, &(r7[i]));
        if (j8[i]) SDL_BlitSurface(imgSelection, NULL, surface, &(r8[i]));
    }

    SDL_Flip(surface);
}


void InitSDL()
{
    atexit(SDL_Quit);

#ifdef TARGET_GP2X
    SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK);
#else
    SDL_Init(SDL_INIT_VIDEO);
#endif

    SDL_ShowCursor(SDL_DISABLE);
    surface = SDL_SetVideoMode(320, 240, 16, SDL_SWSURFACE);

#ifdef TARGET_GP2X
    joystick = SDL_JoystickOpen(0);
#endif

    SDL_Surface* tmpImgBackground = IMG_Load("data/fons.png");
    SDL_SetColorKey(tmpImgBackground, SDL_SRCCOLORKEY, SDL_MapRGB(surface->format, 1, 0, 0));
    imgBackground = SDL_DisplayFormat(tmpImgBackground);

    SDL_Surface* tmpImgSelection = IMG_Load("data/sel.png");
    SDL_SetColorKey(tmpImgSelection, SDL_SRCCOLORKEY, SDL_MapRGB(surface->format, 1, 0, 0));
    imgSelection = SDL_DisplayFormat(tmpImgSelection);
}


void CleanSDL()
{
    SDL_FreeSurface(imgSelection);
    SDL_FreeSurface(imgBackground);

#ifdef TARGET_GP2X
    SDL_JoystickClose(joystick);
#endif
}


#ifdef TARGET_GP2X
void KeyDown(const Uint8& key)
#else
void KeyDown(const SDLKey& key)
#endif
{
    switch(key)
    {
        case GP2X_BUTTON_START:     start=true; break;
        case GP2X_BUTTON_UP:        up=true; break;
        case GP2X_BUTTON_DOWN:      down=true; break;
        case GP2X_BUTTON_LEFT:      left=true; break;
        case GP2X_BUTTON_RIGHT:     right=true; break;
        case GP2X_BUTTON_UPLEFT:    upleft = true; break;
        case GP2X_BUTTON_UPRIGHT:   upright = true; break;
        case GP2X_BUTTON_DOWNLEFT:  downleft = true; break;
        case GP2X_BUTTON_DOWNRIGHT: downright = true; break;
        default: break;
    }
}


#ifdef TARGET_GP2X
void KeyUp(const Uint8& key)
#else
void KeyUp(const SDLKey& key)
#endif
{
    switch(key)
    {
        case GP2X_BUTTON_START:     start=false; break;
        case GP2X_BUTTON_UP:        up=false; break;
        case GP2X_BUTTON_DOWN:      down=false; break;
        case GP2X_BUTTON_LEFT:      left=false; break;
        case GP2X_BUTTON_RIGHT:     right=false; break;
        case GP2X_BUTTON_UPLEFT:    upleft = false; break;
        case GP2X_BUTTON_UPRIGHT:   upright = false; break;
        case GP2X_BUTTON_DOWNLEFT:  downleft = false; break;
        case GP2X_BUTTON_DOWNRIGHT: downright = false; break;
        default: break;
    }
}
